#pragma codeseg HEADFUNC
#define  USE_HY17P68_5M
/******************************************************************************
 * HY17P68WR3_LVD.c                                                        *
 * -------------------------------------------------------------------------- *
 * Copyright 2020 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *                                                                             *
 * For External Input                                                         *
 * IC Body: HY17P68                                                           *

 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>

/**************************************************************************************************|
|*  Static Function                                                                               *|
|**************************************************************************************************/
unsigned char LV17P68WR3(unsigned char AddrH,unsigned char AddrL,unsigned char DataH,unsigned char DataL);
unsigned char EXT17P68WR3(unsigned char AddrH,unsigned char AddrL,unsigned char DataH,unsigned char DataL);
void DelayLIB(unsigned int num);

/**************************************************************************************************|
|*  Generic Peripheral Library                                                                    *|
|**************************************************************************************************/

/*-------------------------------------------------------------------------------------------------|
|  Function: LV17P68WR3                                                                            |
|                                                                                                  |
|  Parameters:                                                                                     |
|         AddrH  -  [in] BIE address high byte                                                     |
|         AddrL   - [in] BIE address low byte                                                      |
|         DataH  - [in] BIE data high byte                                                         |
|         DataH  - [in] BIE data low  byte                                                         |
|  Returns:                                                                                        |
|         None                                                                                     |
|                                                                                                  |
|  Description:                                                                                    |
|                                                                                                  |
|-------------------------------------------------------------------------------------------------*/

unsigned char LV17P68WR3(unsigned char AddrH,unsigned char AddrL,unsigned char DataH,unsigned char DataL)
{
    unsigned char OSCCN0BUF,OSCCN2BUF;
    unsigned char BIEDRHBUF,BIEDRLBUF,BIERLCBUF;
    unsigned char a;
    unsigned char fail= 0;    //0:normal, 1:VDD<2.75V, 2:VPP<8.5V
	int count;

    INTE0 &= 0x7f;         //Disable GIE
    AD1CN0 &= 0x7f;    //Disable ADC
    BIEDRH= 0xff;       //Avoid pumping cause BIE misoperation
    BIEDRL= 0xff;
    
////   Clock setting    //
    OSCCN0BUF= OSCCN0;    //Back up OSCCN setting
    OSCCN2BUF= OSCCN2;
    OSCCN0=0;                         //OSC=HAO
    OSCCN2= 0x03;              //HAOM=01=4.9152MHz, ENHAO=1

////    VDDA setting    //
    PWRCN2 |= 0x04;              //Enable ENLDOPL
	PWRCN= 0b10000110;      //ENBGR+LDOC 2.4V+LDOM VDD+ENLDO
    DelayLIB(1500);
    
////    MFC setting    //                //LVD, Detect VDD>2.75v
    MFCN0= 0b01000010;        //CPRH=VDD+Normal mode
    MFCN1= 0b00101111;        //VSS+22.5R Open+20R Short+V12-RLO
	MFCN2= 0b00011000;        //CPDA=24/32(CPRLH?CPRLL)
    MFCN0 |= 0x01;                  //MFC Enable
    
////  LCD_CK to ext. crystal  //
    OSCCN2 |= 0x20;   //ENXT=1
    __asm__ ("NOP");
    OSCCN1 |= 0x40;    //LCPS=1
    OSCCN1 &= 0xfe;    //LCDS=0
    OSCCN2 &= 0xdf;   //ENXT=0 
    
////    VLCD setting  //
    LCDCN3= 0x00;		//SET  COM0~COM3=IO Mode
    LCDCN4= 0x00;       //LCDCN4=0  SET SEG= IO Mode
    LCDCN5= 0x00;       //LCDCN5=0  
    LCDCN6= 0x00;       //LCDCN6=0  
    LCDCN7= 0x00;       //LCDCN7=0  
    LCDCN8= 0x00;       //LCDCN8=0  
    LCDCN9= 0x00;       //LCDCN9=0  
    LCDCN10= 0x00;      //LCDCN10=0  
    LCDCN11= 0x00;      //LCDCN11=0
////    COM0~COM3 Pull down  //    
    TRISC6 |= 0x0F;      //COM0 Output Low
    PT6 &= 0xF0;
    
    LCDCN1= 0x8B; 
    DelayLIB(2800);                         //DelayLIB 20ms, wait VLCD=5.05V
    BIECN |= 0x14;                      //ENBVD=1, ENBCP=1
    DelayLIB(2800);                        //DelayLIB 20ms, wait VPP=8.5V

	if(MFCN0 & (1 << 5))
	{
	    fail= 1;    //VDD<2.75V;
        goto EndBIE;
    }

    for(count= 0;(BIECN & (1 << 3)) ==0;count++)
	{
	    if(count == 0xffff)
	    {
	        fail= 2;    //VPP<8.5V;
            goto EndBIE;
        }
    }
    BIEARH=AddrH;
    BIEARL=AddrL;
    BIEDRH=DataH;
    BIEDRL=DataL;
    
    BIEDRHBUF=BIEDRH;
	BIEDRLBUF=BIEDRL;

	BIERLCBUF=0xff;
	BIEDRH=0xff;
	for(a=8;a>0;a--)
	{
		BIEARH=AddrH;
		BIEARL=AddrL;
		BIEDRL=BIEDRLBUF;
		BIERLCBUF=BIERLCBUF<<1;
		BIEDRL |= BIERLCBUF;
		BIECN = BIECN | 0x02;
		DelayLIB(650);     //5ms
		for(count= 0;(BIECN & (1 << 3)) ==0;count++)
		{
			if(count == 0xffff)
			{
				fail= 2;    //VPP<8.5V;
				goto EndBIE;
			}
		}
	}

	BIERLCBUF=0xff;
	BIEDRL=BIEDRLBUF;
	for(a=8;a>0;a--)
	{
		BIEARH=AddrH;
		BIEARL=AddrL;
		BIEDRH=BIEDRHBUF;
		BIERLCBUF=BIERLCBUF<<1;
		BIEDRH |= BIERLCBUF;
		BIECN = BIECN | 0x02;
		DelayLIB(650);     //5ms
		for(count= 0;(BIECN & (1 << 3)) ==0;count++)
		{
			if(count == 0xffff)
			{
				fail= 2;    //VPP<8.5V;
				goto EndBIE;
			}
		}
	}
    BIEARH=AddrH;
    BIEARL=AddrL;
    BIEDRH=DataH;
    BIEDRL=DataL;
	BIECN = BIECN | 0x02;
	DelayLIB(650);     //5ms
////    BIE Write Finish  //
    EndBIE: 
    LCDCN1=0;
    DelayLIB(2800);                        //DelayLIB 20ms	
	BIECN &=0xc0;
    if(fail!= 0)   //WRITE FAIL
    {
        BIEDRH=0xff;
        BIEDRL=0xff; 
    }
	OSCCN1 &= 0xbf;    //LCPS=0
	OSCCN2 |= 0x20;   //ENXT=1
    OSCCN2 &= 0xdf;   //ENXT=0
	OSCCN0= OSCCN0BUF;
	OSCCN2= OSCCN2BUF;
    TRISC6 &= 0xF0;
    return fail;

}
unsigned char EXT17P68WR3(unsigned char AddrH,unsigned char AddrL,unsigned char DataH,unsigned char DataL)
{
    unsigned char OSCCN0BUF,OSCCN2BUF;
    unsigned char fail= 0;    //0:normal, 1:VDD<2.75V, 2:VPP<8.5V
	int count;

    INTE0 &= 0x7f;         //Disable GIE
    AD1CN0 &= 0x7f;    //Disable ADC

////   Clock setting    //
    OSCCN0BUF= OSCCN0;    //Back up OSCCN setting
    OSCCN2BUF= OSCCN2;
    OSCCN0=0;                         //OSC=HAO
    OSCCN2= 0x03;              //HAOM=01=4.9152MHz, ENHAO=1
    
////    VDDA setting    //
    PWRCN2 |= 0x04;              //Enable ENLDOPL
	PWRCN= 0b10000110;      //ENBGR+LDOC 2.4V+LDOM VDD+ENLDO
	//PWRCN1= 0b10001011;    //ENREFO+ENVS+0.5*VDDA
    DelayLIB(1500);
    
////  LCD_CK to ext. crystal  //
    OSCCN2 |= 0x20;   //ENXT=1
    __asm__ ("NOP");
    OSCCN1 |= 0x40;    //LCPS=1
    OSCCN1 &= 0xfe;    //LCDS=0
    OSCCN2 &= 0xdf;   //ENXT=0     
////    VLCD setting  //
    LCDCN1= 0x00;       // Disable LCD
    LCDCN3= 0x00;		//SET  COM0~COM3=IO Mode
    LCDCN4= 0x00;       //LCDCN4=0  SET SEG= IO Mode
    LCDCN5= 0x00;       //LCDCN5=0  
    LCDCN6= 0x00;       //LCDCN6=0  
    LCDCN7= 0x00;       //LCDCN7=0  
    LCDCN8= 0x00;       //LCDCN8=0  
    LCDCN9= 0x00;       //LCDCN9=0  
    LCDCN10= 0x00;      //LCDCN10=0  
    LCDCN11= 0x00;      //LCDCN11=0
////    COM0~COM3 Pull down  //    
    TRISC6 |= 0x0F;      //COM0 Output Low
    PT6 &= 0xF0;
    
   
    BIEARH=AddrH;
    BIEARL=AddrL;
    BIEDRH=DataH;
    BIEDRL=DataL;
    BIECN |= 0x10;
    for(count= 0;(BIECN & (1 << 3)) ==0;count++)
	{
	    if(count == 0xffff)
	    {
	        fail= 2;    //VPP<8.5V;
            goto EndBIE;
        }
    }
    BIECN = BIECN | 0x02;
    __asm__ ("NOP");
    while((BIECN & 0x02) ==1);

    EndBIE:
    BIECN &=0xc0;
    if(fail!= 0)   //WRITE FAIL
    {
        BIEDRH=0xff;
        BIEDRL=0xff; 
    }
	OSCCN1 &= 0xbf;    //LCPS=0
	OSCCN2 |= 0x20;   //ENXT=1
    OSCCN2 &= 0xdf;   //ENXT=0
	OSCCN0= OSCCN0BUF;
	OSCCN2= OSCCN2BUF;
    TRISC6 &= 0xF0;
    return fail;
}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Software DelayLIB Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void DelayLIB(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
